################################################################################
# 
# SQL Statement Creation Functions
# 
################################################################################

# concatenate conditions
proc conc_cond {cond_var new_cond {log_op "AND"}} {
	upvar $cond_var cond
	
	# only append if new_cond is not an empty string
	if {$new_cond ne ""} {
		# append the logical operator if the cond string already contains conditions
		if {$cond ne ""} {
			append cond " " $log_op " "
		}
		append cond $new_cond
	}
}

# concatenate tables
proc conc_tables {table_var new_table} {
	upvar $table_var tables
	
	if {$tables ne ""} {
		append tables ", "
	}
	append tables $new_table
}

proc prefix_with_tablename {tablename attributes} {
    
    foreach attr $attributes {
        lappend result [str_concat $tablename "." $attr]
    }
    return $result
}


# returns a list containing the referencing column_name (column of the ref_table)
# and the column_name of the 'referenced table'
proc get_foreign_and_primary_keys {referencing_table_name referenced_table_name} {
	global db_handle
	set t1 $referencing_table_name
	set t2 $referenced_table_name
	
	# This query always returns the column names that are used in a foreign key relation
	set query "SELECT REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME, TABLE_NAME, COLUMN_NAME FROM information_schema.KEY_COLUMN_USAGE where (TABLE_NAME = '$t1' and REFERENCED_TABLE_NAME = '$t2' or TABLE_NAME = '$t2' and REFERENCED_TABLE_NAME = '$t1') and constraint_schema = 'spectral_db'"
	set result [mysqlsel $db_handle $query -flatlist]
	
	#.debug_txt insert end $result
	
	# build the two column names including table prefix
	set col_name1 [str_concat [lindex $result 0] "." [lindex $result 1]]
	set col_name2 [str_concat [lindex $result 2] "." [lindex $result 3]]
	
	# now check which is the referencing table
	if {[lindex $result 0] eq $referencing_table_name} {
		# first table name is the referencing table
		return [list $col_name1 $col_name2]
	} else  {
		return [list $col_name2 $col_name1]
	}
}

proc get_primary_key_column_name {table} {
	global db_handle
	set query "SELECT K.COLUMN_NAME FROM information_schema.KEY_COLUMN_USAGE K where table_name = '$table' and constraint_name = 'PRIMARY'  and constraint_schema = 'spectral_db'"
	set result [mysqlsel $db_handle $query -flatlist]
	return $result
}

# attributes: field names to be selected, e.g. study_name
# tables: tables to be selected from, e.g. study
# conditions: restrictions to select, e.g. study_name like "M%"
# options: stuff like 'order by' or 'limit 10'
proc assemble_sql_query {attributes tables conditions {options ""}} {
	set query "SELECT "
	append query $attributes
	append query " FROM "
	append query $tables
	if {$conditions ne ""} {
		append query " WHERE "
		append query $conditions
	}
	append query " " $options
	
	return $query
}

# returns the datatype of each column
# plus a list with the length of the varchar columns
# for non-varchar columns '0' will be inserted
proc get_datatype_and_varchar_length {table column_names} {
    global db_handle
    set result {}
    
    # get lengthes
    foreach col $column_names {
        set query "SELECT C.DATA_TYPE, C.CHARACTER_MAXIMUM_LENGTH FROM information_schema.`COLUMNS` C where table_schema = 'spectral_db' and table_name = '$table' and column_name = '$col'"
        set res [mysqlsel $db_handle $query -flatlist]
        if {[lindex  $res 0] eq "varchar"} {
            lappend result [lindex $res 0] [lindex $res 1]
        } else  {
            lappend result [lindex $res 0] 0
        }
    }
    
    return $result    
}

